#!/bin/sh
#
# Version 24107
#
# You can add this script to your cron tab to periodically update the Reactor System entity
# with some Pi-specific measurements (stored in extended attributes).
#
# ***NOTE*** If you have user authentication enabled, this script will not work.
# Please see the documentation for user authentication for information.
#
# Ref: https://www.raspberrypi.org/documentation/raspbian/applications/vcgencmd.md
#

LOG=/tmp/rpi-status.log

# System temperature
temp=$( vcgencmd measure_temp | sed -e "s/^.*=\(.*\)'.*/\\1/" )
curl -s -q -o - \
    "http://127.0.0.1:8111/api/v1/entity/reactor_system>system/attribute/x_reactor_system_extra.sys_temp/set?value=$temp&type=real" >${LOG} 2>&1

# CPU core voltage (normally 0.85 on my RPI4)
temp=$( vcgencmd measure_volts core | sed -e "s/^.*=\(.*\)V.*/\\1/" )
curl -s -q -o - \
    "http://127.0.0.1:8111/api/v1/entity/reactor_system>system/attribute/x_reactor_system_extra.sys_cpu_volts/set?value=$temp&type=real" >>${LOG} 2>&1

# Throttling (ref: https://www.raspberrypi.com/documentation/computers/os.html#vcgencmd)
temp=$( vcgencmd get_throttled | sed -e "s/^.*=\(.*\)V.*/\\1/" )
curl -s -q -o - \
    "http://127.0.0.1:8111/api/v1/entity/reactor_system>system/attribute/x_reactor_system_extra.sys_throttled/set?value=$temp&type=int" >>${LOG} 2>&1

# Other things/queries we could do: "measure_volts sdram_[cip]" (core, I/O, phy), "measure_clock (arm|core)"